@@ -210,7 +210,11 @@ fn test_eu4_ita1() {
 #[test]
 fn test_roundtrip_melt() {
     let data = utils::request("kandy2.bin.eu4");
-    let (out, _unknown) = eu4save::melt(&data[..], eu4save::FailedResolveStrategy::Error).unwrap();
+    let (out, _unknown) = eu4save::Melter::new()
+        .with_on_failed_resolve(FailedResolveStrategy::Error)
+        .melt(&data[..])
+        .unwrap();
+
     let (save, encoding) = Eu4Extractor::extract_save(Cursor::new(&out[..])).unwrap();
     assert_eq!(encoding, Encoding::Text);
     assert_eq!(save.meta.player, "BHA".parse().unwrap());
@@ -226,7 +230,11 @@ macro_rules! ironman_test {
                 // Ensure that every ironman can be melted with all tokens resolvable.
                 // Deserialization will not try and resolve tokens that aren't used. Melting
                 // ensures that every token is seen
-                let (melted, _) = eu4save::melt(&data[..], FailedResolveStrategy::Error).unwrap();
+                let (melted, _) = eu4save::Melter::new()
+                    .with_on_failed_resolve(FailedResolveStrategy::Error)
+                    .melt(&data[..])
+                    .unwrap();
+
                 assert!(!melted.is_empty());
 
                 let (save, encoding) = Eu4ExtractorBuilder::new()
@@ -892,5 +900,6 @@ ironman_test!(
     |_query: Query, melted_data: &[u8]| {
         // Find inukshuk
         twoway::find_bytes(melted_data, b"date_built=-2000.1.1").unwrap();
+        twoway::find_bytes(melted_data, b"navy_strength=0.00000").unwrap();
     }
 );
